@@ -7,7 +7,10 @@ import datetime
 
 from uuid import uuid4
 from jsonschema.validators import Draft4Validator
-from six.moves.urllib.parse import urlparse
+from six.moves.urllib.parse import urlparse, urlunparse, unquote
+from base64 import b64decode
+
+from rabix.common.errors import ValidationError
 
 
 log = logging.getLogger(__name__)
@@ -48,9 +51,10 @@ class App(object):
                 return False
         return True
 
-    def job_dump(self, job, dirname, context):
+    def job_dump(self, job, dirname):
         with open(os.path.join(dirname, 'job.cwl.json'), 'w') as f:
-            json.dump(job.to_dict(context), f)
+            job_dict = job.to_dict()
+            json.dump(job_dict, f)
             log.info('File %s created.', job.id)
 
     def to_dict(self, context):
@@ -65,43 +69,145 @@ class App(object):
         }
 
 
+class URL(object):
+
+    def __init__(self, url):
+        (self.scheme, self.netloc, self.path,
+         self.params, self.query, self.fragment) = urlparse(url, 'file')
+
+        self.content_type = None
+        self.charset = None
+        self.data = None
+        if self.scheme == 'data':
+            meta, data = self.path.split(',')
+            meta_parts = meta.split(';')
+            self.content_type = meta_parts[0]
+            b64 = 'base64' in meta_parts
+            if b64:
+                self.data = b64decode(data)
+            else:
+                self.data = unquote(data)
+
+    def islocal(self):
+        return self.scheme == 'file'
+
+    def geturl(self):
+        return urlunparse(
+            (self.scheme, self.netloc, self.path,
+             self.params, self.query, self.fragment)
+        )
+
+    def isdata(self):
+        return self.scheme == 'data'
+
+    def __str__(self):
+        if self.islocal():
+            return self.path
+        else:
+            return self.geturl()
+
+
 class File(object):
 
     def __init__(self, path, size=None, meta=None, secondary_files=None):
-        self.url = path
+        self.url = URL(path) if isinstance(path, six.string_types) else path
         self.size = size
         self.meta = meta or {}
         self.secondary_files = secondary_files or []
 
-    def to_dict(self, context):
-        if isinstance(self.url, str):
-            path = self.url
-        else:
-            path = self.url.geturl()
+    def to_dict(self, context=None):
         return {
             "@type": "File",
-            "path": path,
+            "path": self.path,
             "size": self.size,
             "metadata": self.meta,
             "secondaryFiles": [sf.to_dict(context)
                                for sf in self.secondary_files]
         }
 
-    @classmethod
-    def from_dict(cls, d):
+    @property
+    def path(self):
+        return six.text_type(self.url)
 
-        if isinstance(d, six.string_types):
-            d = {'path': d}
+    def __str__(self):
+        return self.path
 
-        size = d.get('size')
-        if size is not None:
-            size = int(size)
+    def __repr__(self):
+        return "File(" + six.text_type(self.to_dict()) + ")"
 
-        return cls(path=d.get('path'),
-                   size=size,
-                   meta=d.get('meta'),
-                   secondary_files=[File.from_dict(sf)
-                                    for sf in d.get('secondaryFiles', [])])
+    @path.setter
+    def path(self, val):
+        self.url = val
+
+
+def make_constructor(schema):
+        constructor_map = {
+            'integer': int,
+            'number': float,
+            'boolean': bool,
+            'string': six.text_type,
+            'file': FileConstructor,
+            'directory': FileConstructor
+        }
+
+        type_name = schema.get('type')
+
+        if not type_name:
+            return lambda x: x
+
+        if type_name == 'array':
+            item_constructor = make_constructor(schema.get('items', {}))
+            return ArrayConstructor(item_constructor)
+
+        if type_name == 'object':
+            return ObjectConstructor(schema.get('properties', {}))
+
+        return constructor_map[type_name]
+
+
+class ArrayConstructor(object):
+
+    def __init__(self, item_constructor):
+        self.item_constructor = item_constructor
+
+    def __call__(self, val):
+        return [self.item_constructor(v) for v in val]
+
+
+class ObjectConstructor(object):
+
+    def __init__(self, properties):
+        self.properties = {
+            k: make_constructor(v)
+            for k, v in six.iteritems(properties)
+        }
+
+    def __call__(self, val):
+        return {
+            k: self.properties.get(k, lambda x: x)(v)
+            for k, v in six.iteritems(val)
+        }
+
+
+def FileConstructor(val):
+    if isinstance(val, six.string_types):
+        val = {'path': val}
+
+    size = val.get('size')
+    if size is not None:
+        size = int(size)
+
+    path = val.get('path')
+    if path is None:
+        raise ValidationError(
+            "Not a valid 'File' object: %s" % str(val)
+        )
+
+    return File(path=path,
+                size=size,
+                meta=val.get('meta'),
+                secondary_files=[FileConstructor(sf)
+                                 for sf in val.get('secondaryFiles', [])])
 
 
 class IO(object):
@@ -130,15 +236,7 @@ class IO(object):
 
     @classmethod
     def from_dict(cls, context, d):
-        constructor_map = {
-            'integer': int,
-            'number': float,
-            'boolean': bool,
-            'object': dict,
-            'string': str,
-            'file': File.from_dict,
-            'directory': File.from_dict
-        }
+
         item_schema = d.get('schema', {})
         type_name = item_schema.get('type')
         depth = 0
@@ -147,9 +245,11 @@ class IO(object):
             item_schema = item_schema.get('items', {})
             type_name = item_schema.get('type')
 
+        constructor = make_constructor(item_schema)
+
         return cls(d.get('@id', str(uuid4())),
                    validator=context.from_dict(d.get('schema')),
-                   constructor=constructor_map[type_name],
+                   constructor=constructor,
                    required=d['required'],
                    annotations=d['annotations'],
                    depth=depth)
@@ -157,24 +257,27 @@ class IO(object):
 
 class Job(object):
 
-    def __init__(self, job_id, app, inputs, allocated_resources):
+    def __init__(self, job_id, app, inputs, allocated_resources, context):
         # if not app.validate_inputs(inputs):
         #     raise ValidationError("Invalid inputs for application %s" % app.id)
         self.id = job_id
         self.app = app
         self.inputs = inputs
         self.allocated_resources = allocated_resources
+        self.context = context
+        pass
 
     def run(self):
         return self.app.run(self)
 
-    def to_dict(self, context):
+    def to_dict(self, context=None):
+        ctx = context or self.context
         return {
             '@id': self.id,
             '@type': 'Job',
-            'app': self.app.to_dict(context),
-            'inputs': context.to_dict(self.inputs),
-            'allocatedResources': self.allocated_resources
+            'app': self.app.to_dict(ctx),
+            'inputs': ctx.to_dict(self.inputs),
+            'allocatedResources': ctx.to_dict(self.allocated_resources)
         }
 
     @staticmethod
@@ -194,8 +297,11 @@ class Job(object):
     def from_dict(cls, context, d):
         app = context.from_dict(d['app'])
         return cls(
-            d.get('@id') if d.get('@id') else cls.mk_work_dir(app.id), app,
-            context.from_dict(d['inputs']), d.get('allocatedResources')
+            d.get('@id') if d.get('@id') else cls.mk_work_dir(app.id),
+            app,
+            context.from_dict(d['inputs']),
+            d.get('allocatedResources'),
+            context
         )
 
 
